#ifndef IniFileH
#define	IniFileH
//------------------------------------------------------------------------------
#include <GraphicsDefs.h>
#include <List.h>
#include <SupportDefs.h>
#include <String.h>
#include <stdio.h>
#include <string.h>
#include <strstream.h>
#include "SHIFile.h"
#include "SHIString.h"
//------------------------------------------------------------------------------
const char	GlobalSection[] = "_TIniF_GlobalSec";
//const char	ReadValueErr[] = "_TIniF_RValueErr";
const int	IniFileMaxLen = 2048;
//------------------------------------------------------------------------------
class	TIniFileSection;
//------------------------------------------------------------------------------
class	TIniFile : public TSHIFile
{
public:
	TIniFile(int iMaxLen = IniFileMaxLen);
	TIniFile(const entry_ref *ref,uint32 open_mode,int iMaxLen = IniFileMaxLen);
	TIniFile(const BEntry *entry,uint32 open_mode,int iMaxLen = IniFileMaxLen);
	TIniFile(const char *path,uint32 open_mode,int iMaxLen = IniFileMaxLen);
	TIniFile(const BDirectory *dir,const char *path,uint32 open_mode,int iMaxLen = IniFileMaxLen);
	TIniFile(const BFile &file,int iMaxLen = IniFileMaxLen);
	virtual ~TIniFile();

	virtual bool		ChangeSection(const char *cSecName);			//アクティブセクションの変更
	virtual bool		DeleteKey(const char *cKey);					//キーの削除
	virtual bool		DeleteSection(const char *cSecName);			//セクションの削除
	virtual void		EnumKeys(BList *xList);							//キーの列挙（アイテムの開放・変更厳禁）
	virtual void		EnumSections(BList *xList);						//セクションの列挙（アイテムの開放・変更厳禁）
	virtual bool		Flush(bool bForce = false);						//実際の書き込み
	virtual bool		KeyExists(const char *cKey);					//キーが存在するか
	virtual bool		ReadComment(char *cComment);					//コメントの読み込み
	virtual bool		ReadValue(const char *cKey,char *cValue);		//価の読み込み
	virtual bool		ReadValue(const char *cKey,bool *bValue);		//価の読み込み
	virtual bool		ReadValue(const char *cKey,int32 *iValue);		//価の読み込み
	virtual bool		ReadValue(const char *cKey,int64 *iValue);		//価の読み込み
	virtual bool		ReadValue(const char *cKey,float *fValue);		//価の読み込み
	virtual bool		ReadValue(const char *cKey,rgb_color *iValue);	//価の読み込み
//template <class T> bool ReadValue(const char *cKey,T *x);
//bool	ReadValue(const char *cKey,int);
//	template <class T>	bool ReadValue(const char *cKey,T *xValue);		//価の読み込み
//	/*virtual*/ bool		ReadValue(const char *,int32);
//	/*virtual*/ bool		ReadValue(const char *,int64);
	virtual bool		SectionExists(const char *cSecName);				//セクションが存在するか
	virtual	status_t	SetTo(const entry_ref *ref,uint32 openMode,int iMaxLen = IniFileMaxLen);
	virtual	status_t	SetTo(const BEntry *entry,uint32 openMode,int iMaxLen = IniFileMaxLen);
	virtual	status_t	SetTo(const char *path,uint32 openMode,int iMaxLen = IniFileMaxLen);
	virtual	status_t	SetTo(const BDirectory *dir,const char *path,uint32 openMode,int iMaxLen = IniFileMaxLen);
	virtual void		Unset();
	virtual void		WriteComment(const char *cComment);					//コメントの書き込み
	virtual void		WriteValue(const char *cKey,const char *cValue);	//価の書き込み
	virtual void		WriteValue(const char *cKey,bool bValue);			//価の書き込み
	virtual void		WriteValue(const char *cKey,int32 iValue);			//価の書き込み
	virtual void		WriteValue(const char *cKey,int64 iValue);			//価の書き込み
	virtual void		WriteValue(const char *cKey,float fValue);			//価の書き込み
	virtual void		WriteValue(const char *cKey,rgb_color xValue);		//価の書き込み
private:
	TIniFileSection		*CreateSection(const char *cSecName,int iIdx = -1);	//セクション作成・追加
	void				DestroyVar();										//変数の破棄
	TIniFileSection		*FindSection(const char *cSecName);					//セクションを探す
//	int					GetSectionIdx(const char *cSecName);					//セクションリスト中の番号
	void				Init(int);											//初期化
	void				Load();												//読み込み

	bool			mbModified;			//内容が変更された
	int				miMaxLen;			//１行の最大文字数
	BList			mxSections;			//セクション群
	TIniFileSection	*mxSecCurrent;		//アクティブセクション
};
//==============================================================================
class	TIniFilePair
{
public:
	TIniFilePair(int iMaxLen);
	~TIniFilePair();
	
	char	*mcKey;		//キー
	char	*mcValue;	//価
};
//==============================================================================
class	TIniFileSection
{
public:
	TIniFileSection(int iMaxLen);
	~TIniFileSection();
	
//	void			AddComment(BString xAddComment);				//コメントの追加
	void			AddComment(const char *cAddComment);	//コメントの追加
	TIniFilePair	*CreatePair(const char *cKey,const char *cValue = "");	//ペア作成・追加
	bool			DeletePair(const char *cKey);			//ペア削除
	void			EnumKeys(BList *xPair);					//キーの列挙
	TIniFilePair	*FindPair(const char *cKey);			//キーからペアを探す
//	bool			ReadComment(BString &xComment);				//コメントの読みだし
	bool			ReadComment(char *cComment);			//コメントの読みだし
	
	char	*mcName;			//セクション名
private:
	friend class TIniFile;
	
	int		miCommentPos;	//コメント読みだしの位置
	int		miMaxLen;		//１行の最大文字数
	BList	mxPairs;		//ペア群
	BString	mxComment;		//コメント
};
//------------------------------------------------------------------------------
#endif	//IniFileH
